@@ -21,6 +21,7 @@ from pprint import pformat
 from dlquery.collection import Tabular
 
 from templateapp import TemplateBuilder
+from templateapp.exceptions import TemplateBuilderInvalidFormat
 from templateapp.core import save_file
 
 from templateapp import version
@@ -421,7 +422,7 @@ class Application:
 
         self._base_title = 'TemplateApp {}'.format(edition)
         self.root = tk.Tk()
-        self.root.geometry('800x600+100+100')
+        self.root.geometry('900x600+100+100')
         self.root.minsize(200, 200)
         self.root.option_add('*tearOff', False)
 
@@ -449,6 +450,10 @@ class Application:
         self.result_btn = None
         self.store_btn = None
         self.search_chkbox = None
+        self.template_name_txtbox = None
+        self.lookup_btn = None
+        self.close_lookup_btn = None
+        self.close_backup_btn = None
 
         self.curr_widget = None
         self.prev_widget = None
@@ -463,9 +468,12 @@ class Application:
         self.snapshot.update(result='')
         self.snapshot.update(template='')
         self.snapshot.update(is_built=False)
+        self.snapshot.update(curr_app='main_app')
         self.snapshot.update(switch_app_template='')
         self.snapshot.update(switch_app_user_data='')
         self.snapshot.update(switch_app_result_data='')
+        self.snapshot.update(main_input_textarea='')
+        self.snapshot.update(main_result_textarea='')
 
         # variables
         self.build_btn_var = tk.StringVar()
@@ -589,6 +597,7 @@ class Application:
 
     def shift_to_main_app(self):
         """Switch from backup app to main app"""
+        self.snapshot.update(curr_app='main_app')
         user_data = self.snapshot.switch_app_user_data      # noqa
         result_data = self.snapshot.switch_app_result_data  # noqa
         self.snapshot.update(switch_app_user_data='')
@@ -600,16 +609,17 @@ class Application:
 
         stored_title = self.root.title().replace(' - ' + self._base_title, '')
         self.snapshot.update(stored_title=stored_title)
-        self.set_title(title=self.snapshot.title)
+        self.set_title(title=self.snapshot.title)   # noqa
 
     def shift_to_backup_app(self):
         """Switch from main app to backup app"""
+        self.snapshot.update(curr_app='backup_app')
         self.paned_window.remove(self.entry_frame)
         self.paned_window.insert(1, self.backup_frame)
 
         title = self.root.title().replace(' - ' + self._base_title, '')
         self.snapshot.update(title=title)
-        stored_title = self.snapshot.stored_title or 'Storing Template'
+        stored_title = self.snapshot.stored_title or 'Storing Template'     # noqa
         self.set_title(title=stored_title)
 
     def callback_focus(self, event):
@@ -635,15 +645,64 @@ class Application:
         filename = filedialog.askopenfilename(filetypes=filetypes)
         if filename:
             with open(filename) as stream:
+                if self.search_chkbox_var.get():
+                    self.search_chkbox.invoke()
+
+                if self.snapshot.curr_app == 'backup_app':
+                    self.close_backup_btn.invoke()
+
                 content = stream.read()
                 self.test_data_btn.config(state=tk.NORMAL)
                 self.test_data_btn_var.set('Test Data')
                 self.set_textarea(self.result_textarea, '')
                 self.snapshot.update(test_data=content)
                 title = 'Open {} + LOAD Test Data'.format(filename)
-                self.set_textarea(self.input_textarea, content, title=title)
+                self.set_title(title=title)
+                self.set_textarea(self.input_textarea, content)
+                self.copy_text_btn.configure(state=tk.NORMAL)
+                self.save_as_btn.configure(state=tk.NORMAL)
                 self.input_textarea.focus()
 
+    def callback_load_td_file(self):
+        """Callback for Menu File > Load Test Data."""
+        filetypes = [
+            ('Text Files', '.txt', 'TEXT'),
+            ('All Files', '*'),
+        ]
+        filename = filedialog.askopenfilename(filetypes=filetypes)
+        if filename:
+            with open(filename) as stream:
+                if self.search_chkbox_var.get():
+                    self.search_chkbox.invoke()
+
+                if self.snapshot.curr_app == 'backup_app':
+                    self.close_backup_btn.invoke()
+
+                content = stream.read()
+                self.test_data_btn.config(state=tk.NORMAL)
+                self.test_data_btn_var.set('Test Data')
+
+                input_data = Application.get_textarea(self.input_textarea)
+                result_data = Application.get_textarea(self.result_textarea)
+                if content.strip() == input_data.strip() or input_data.strip() == '':
+                    self.set_textarea(self.input_textarea, content)
+                    if input_data.strip() == '':
+                        pattern = r'#+\s+# *Template +is +generated '
+                        if not re.match(pattern, result_data):
+                            self.set_textarea(self.result_textarea, '')
+                        else:
+                            self.result_btn.configure(state=tk.NORMAL)
+                    self.input_textarea.focus()
+                else:
+                    self.set_textarea(self.result_textarea, content)
+
+                self.copy_text_btn.configure(state=tk.NORMAL)
+                self.save_as_btn.configure(state=tk.NORMAL)
+                self.snapshot.update(test_data=content)
+                title = 'LOAD Test Data - {}'.format(filename)
+                self.snapshot.update(title=title)
+                self.set_title(title=title)
+
     def callback_help_documentation(self):
         """Callback for Menu Help > Getting Started."""
         self.browser.open_new_tab(Data.documentation_url)
@@ -872,6 +931,11 @@ class Application:
 
         set_modal_dialog(settings)
 
+    def callback_preferences_user_template(self):
+        """Callback for Menu Preferences > User Template"""
+        self.search_chkbox_var.set(False)
+        self.search_chkbox.invoke()
+
     def build_menu(self):
         """Build menubar for Regex GUI."""
         menu_bar = tk.Menu(self.root)
@@ -885,6 +949,7 @@ class Application:
         menu_bar.add_cascade(menu=help_, label='Help')
 
         file.add_command(label='Open', command=lambda: self.callback_open_file())
+        file.add_command(label='Load Test Data', command=lambda: self.callback_load_td_file())
         file.add_separator()
         file.add_command(label='Quit', command=lambda: self.callback_file_exit())
 
@@ -892,7 +957,11 @@ class Application:
             label='Settings',
             command=lambda: self.callback_preferences_settings()
         )
-        # preferences.add_separator()
+        preferences.add_separator()
+        preferences.add_command(
+            label='User Template',
+            command=lambda: self.callback_preferences_user_template()
+        )
 
         help_.add_command(label='Documentation',
                           command=lambda: self.callback_help_documentation())
@@ -948,64 +1017,50 @@ class Application:
         """Build input entry for regex GUI."""
 
         def callback_build_btn():
-            if self.build_btn_var.get() == 'Build':
-                user_data = Application.get_textarea(self.input_textarea)
-                if not user_data:
-                    create_msgbox(
-                        title='Empty Data',
-                        error="Can NOT build regex pattern without data."
-                    )
-                    return
+            user_data = Application.get_textarea(self.input_textarea)
+            if not user_data:
+                create_msgbox(
+                    title='Empty Data',
+                    error="Can NOT build regex pattern without data."
+                )
+                return
 
-                try:
-                    kwargs = self.get_template_args()
-                    factory = TemplateBuilder(user_data=user_data, **kwargs)
-                    self.snapshot.update(user_data=user_data)
-                    self.snapshot.update(result=factory.template)
-                    self.snapshot.update(template=factory.template)
-                    self.snapshot.update(swich_app_template='')
-                    self.snapshot.update(is_built=True)
-                    self.test_data_btn_var.set('Test Data')
-                    self.save_as_btn.config(state=tk.NORMAL)
-                    self.copy_text_btn.config(state=tk.NORMAL)
-                    self.set_textarea(self.result_textarea, factory.template)
+            try:
+                kwargs = self.get_template_args()
+                factory = TemplateBuilder(user_data=user_data, **kwargs)
+                self.snapshot.update(user_data=user_data)
+                self.snapshot.update(result=factory.template)
+                self.snapshot.update(template=factory.template)
+                self.snapshot.update(swich_app_template='')
+                self.snapshot.update(is_built=True)
+                self.test_data_btn_var.set('Test Data')
+                self.save_as_btn.config(state=tk.NORMAL)
+                self.copy_text_btn.config(state=tk.NORMAL)
+                self.set_textarea(self.result_textarea, factory.template)
 
-                    title = 'Building Template'
-                    self.snapshot.update(title=title)
-                    self.set_title(title=title)
-                except Exception as ex:
-                    error = '{}: {}'.format(type(ex).__name__, ex)
-                    create_msgbox(title='RegexBuilder Error', error=error)
+                title = 'Building Template'
+                self.snapshot.update(title=title)
+                self.set_title(title=title)
+            except TemplateBuilderInvalidFormat as ex:
+                error = '{}: {}'.format(type(ex).__name__, ex)
+                create_msgbox(title='RegexBuilder Error', error=error)
+            except Exception as ex:
+                error = '{}: {}'.format(type(ex).__name__, ex)
+                create_msgbox(title='RegexBuilder Error', error=error)
+                kwargs = self.get_template_args()
+                factory = TemplateBuilder(user_data=user_data, debug=True, **kwargs)
+                fmt = '# Please fix user_data to produce a good template\n{}'
+                content = fmt.format(factory.bad_template)
+                self.set_textarea(self.result_textarea, content)
+                title = 'Bad Generated Template'
+                self.snapshot.update(title=title)
+                self.set_title(title=title)
 
-                if self.snapshot.template:  # noqa
-                    self.store_btn.config(state=tk.NORMAL)
+            if self.snapshot.template:  # noqa
+                self.store_btn.config(state=tk.NORMAL)
 
-                if self.snapshot.is_built:  # noqa
-                    self.result_btn.config(state=tk.NORMAL)
-            else:
-                template_name = self.template_name_var.get().strip()
-                if template_name:
-                    user_template = UserTemplate()
-                    template = user_template.search(template_name)
-                    self.result_btn.config(state=tk.NORMAL)
-                    self.save_as_btn.config(state=tk.NORMAL)
-                    self.copy_text_btn.config(state=tk.NORMAL)
-                    self.test_data_btn_var.set('Test Data')
-                    if template:
-                        self.snapshot.update(template=template)
-                        self.snapshot.update(result=template)
-                        self.set_textarea(self.result_textarea, template)
-                    else:
-                        self.snapshot.update(result=user_template.status)
-                        self.set_textarea(self.result_textarea, user_template.status)
-
-                    title = 'Searching Template'
-                    self.snapshot.update(title=title)
-                    self.set_title(title=title)
-                else:
-                    title = 'Empty Template Name'
-                    error = 'CANT retrieve template with empty template name.'
-                    create_msgbox(title=title, error=error)
+            if self.snapshot.is_built:  # noqa
+                self.result_btn.config(state=tk.NORMAL)
 
         def callback_save_as_btn():
             prev_widget_name = str(self.prev_widget)
@@ -1066,7 +1121,17 @@ class Application:
                             if response:
                                 node = node.with_name(new_name)
                 filename = str(node)
-                save_file(filename, content)
+                if not content.strip():
+                    fmt = ('The content of {} is empty.\n'
+                           'Do you want save the empty file?')
+                    question = fmt.format(filename)
+                    title = '{} - {}'.format(title, 'Empty')
+                    response = create_msgbox(question=question, title=title)
+                else:
+                    response = 'yes'
+
+                if response == 'yes':
+                    save_file(filename, content)
 
         def callback_clear_text_btn():
             prev_widget_name = str(self.prev_widget)
@@ -1095,6 +1160,7 @@ class Application:
                     self.test_data_btn.config(state=tk.DISABLED)
                     self.result_btn.config(state=tk.DISABLED)
                     self.store_btn.config(state=tk.DISABLED)
+                    self.input_textarea.config(state=tk.NORMAL)
 
                     self.snapshot.update(user_data='')
                     self.snapshot.update(test_data=None)
@@ -1182,6 +1248,9 @@ class Application:
                     self.set_textarea(self.input_textarea, data)
                     self.input_textarea.focus()
 
+                self.copy_text_btn.configure(state=tk.NORMAL)
+                self.save_as_btn.configure(state=tk.NORMAL)
+
                 self.snapshot.update(title=title)
                 self.set_title(title=title)
 
@@ -1325,7 +1394,7 @@ class Application:
                 )
             else:
                 self.test_data_btn_var.set('Test Data')
-                self.set_title(title=self.snapshot.title)
+                self.set_title(title=self.snapshot.title)   # noqa
                 self.set_textarea(
                     self.result_textarea,
                     self.snapshot.result  # noqa
@@ -1341,26 +1410,24 @@ class Application:
                 )
                 return
 
-            if self.build_btn_var.get() == 'Search':
-                template = self.snapshot.template   # noqa
+            user_data = Application.get_textarea(self.input_textarea)
+            if not user_data:
+                create_msgbox(
+                    title='Empty Data',
+                    error="Can NOT build regex pattern without data."
+                )
+                return
 
-            if self.build_btn_var.get() == 'Build':
-                user_data = Application.get_textarea(self.input_textarea)
-                if not user_data:
-                    create_msgbox(
-                        title='Empty Data',
-                        error="Can NOT build regex pattern without data."
-                    )
-                    return
-
-                try:
-                    kwargs = self.get_template_args()
-                    factory = TemplateBuilder(user_data=user_data, **kwargs)
-                    self.snapshot.update(user_data=user_data)
-                    self.snapshot.update(template=factory.template)
-                    self.snapshot.update(is_built=True)
-                    template = factory.template
-                except Exception as ex:
+            try:
+                kwargs = self.get_template_args()
+                factory = TemplateBuilder(user_data=user_data, **kwargs)
+                self.snapshot.update(user_data=user_data)
+                self.snapshot.update(template=factory.template)
+                self.snapshot.update(is_built=True)
+                template = factory.template
+            except Exception as ex:
+                template = self.snapshot.template.strip()
+                if not template:
                     error = '{}: {}'.format(type(ex).__name__, ex)
                     create_msgbox(title='RegexBuilder Error', error=error)
                     return
@@ -1435,26 +1502,110 @@ class Application:
                 return
 
             if self.search_chkbox_var.get():
-                self.build_btn_var.set('Search')
+                self.open_file_btn.configure(state=tk.DISABLED)
+                self.copy_text_btn.configure(state=tk.DISABLED)
+                self.save_as_btn.configure(state=tk.DISABLED)
+                self.paste_text_btn.configure(state=tk.DISABLED)
+                self.clear_text_btn.configure(state=tk.DISABLED)
+                self.build_btn.configure(state=tk.DISABLED)
                 self.snippet_btn.configure(state=tk.DISABLED)
                 self.unittest_btn.configure(state=tk.DISABLED)
                 self.pytest_btn.configure(state=tk.DISABLED)
+                self.result_btn.config(state=tk.DISABLED)
+                self.test_data_btn.config(state=tk.DISABLED)
                 self.store_btn.configure(state=tk.DISABLED)
 
+                self.input_textarea.configure(state=tk.DISABLED)
+                self.lookup_btn.grid(row=0, column=2, sticky=tk.W)
+                self.close_lookup_btn.grid(row=0, column=3, sticky=tk.W)
+                self.template_name_txtbox.focus()
+
+                input_txt = Application.get_textarea(self.input_textarea)
+                result_txt = Application.get_textarea(self.result_textarea)
+                self.snapshot.update(main_input_textarea=input_txt)
+                self.snapshot.update(main_result_textarea=result_txt)
+
+                template_name = self.template_name_var.get().strip()
+                if template_name:
+                    template = user_template.search(template_name)
+                    if template:
+                        self.snapshot.update(template=template)
+                        self.snapshot.update(result=template)
+                        self.set_textarea(self.input_textarea, template)
+                else:
+                    self.set_textarea(self.input_textarea, '')
+
+                self.set_textarea(self.result_textarea, user_template.read())
+
                 title = self.root.title().replace(' - ' + self._base_title, '')
                 self.snapshot.update(title=title)
                 self.set_title(title='Searching Template')
             else:
-                self.build_btn_var.set('Build')
+                self.open_file_btn.configure(state=tk.NORMAL)
+                # self.copy_text_btn.configure(state=tk.NORMAL)
+                # self.save_as_btn.configure(state=tk.NORMAL)
+                self.paste_text_btn.configure(state=tk.NORMAL)
+                self.clear_text_btn.configure(state=tk.NORMAL)
+                self.build_btn.configure(state=tk.NORMAL)
                 self.snippet_btn.configure(state=tk.NORMAL)
                 self.unittest_btn.configure(state=tk.NORMAL)
                 self.pytest_btn.configure(state=tk.NORMAL)
-                self.store_btn.configure(state=tk.NORMAL)
 
-                title_ = self.snapshot.title
-                title = title_ if title_ != 'Searching Template' else ''
+                if self.snapshot.test_data:     # noqa
+                    self.result_btn.config(state=tk.NORMAL)
+                    self.test_data_btn.config(state=tk.NORMAL)
+
+                if self.snapshot.is_built:      # noqa
+                    self.store_btn.configure(state=tk.NORMAL)
+
+                self.input_textarea.configure(state=tk.NORMAL)
+                self.lookup_btn.grid_forget()
+                self.close_lookup_btn.grid_forget()
+
+                input_txt = Application.get_textarea(self.input_textarea)
+                pattern = r'#+\s+# *Template +is +generated '
+                if re.match(pattern, input_txt):
+                    self.test_data_btn_var.set('Test Data')
+                    self.set_textarea(self.result_textarea, input_txt)
+                else:
+                    self.set_textarea(
+                        self.result_textarea,
+                        self.snapshot.main_result_textarea,     # noqa
+                    )
+
+                self.set_textarea(
+                    self.input_textarea,
+                    self.snapshot.main_input_textarea,  # noqa
+                )
+
+                input_txt = Application.get_textarea(self.input_textarea)
+                result_txt = Application.get_textarea(self.result_textarea)
+
+                if input_txt or result_txt:
+                    self.copy_text_btn.configure(state=tk.NORMAL)
+                    self.save_as_btn.configure(state=tk.NORMAL)
+
+                title_ = self.snapshot.title    # noqa
+                title = '' if title_ == self._base_title else title_
                 self.snapshot.update(title=title)
-                self.set_title(title=title)
+                self.set_title(title=title) if title else self.set_title()
+
+        def callback_lookup_btn():
+            template_name = self.template_name_var.get().strip()
+            if template_name:
+                user_template = UserTemplate()
+                template = user_template.search(template_name)
+                if template:
+                    self.snapshot.update(template=template)
+                    self.snapshot.update(result=template)
+                    self.set_textarea(self.input_textarea, template)
+                else:
+                    self.snapshot.update(result=user_template.status)
+                    self.set_textarea(self.input_textarea, user_template.status)
+            else:
+                title = 'Empty Template Name'
+                error = 'CANT retrieve template with empty template name.'
+                create_msgbox(title=title, error=error)
 
         def callback_app_backup_refresh_btn():
             user_data = self.snapshot.switch_app_user_data      # noqa
@@ -1525,8 +1676,6 @@ class Application:
         )
         self.save_as_btn.grid(row=0, column=1, pady=(2, 0))
 
-        # customize width for buttons
-        btn_width = 5.5 if self.is_macos else 8
         # copy button
         self.copy_text_btn = self.Button(
             self.entry_frame, text='Copy',
@@ -1603,8 +1752,6 @@ class Application:
         )
         self.test_data_btn.grid(row=0, column=9, pady=(2, 0))
 
-        # customize width for buttons
-        btn_width = 6 if self.is_macos else 8
         # result button
         self.result_btn = self.Button(
             self.entry_frame, text='Result',
@@ -1642,11 +1789,26 @@ class Application:
         self.search_chkbox.grid(row=0, column=0, padx=(0, x), sticky=tk.W)
 
         # template name textbox
-        self.TextBox(
-            frame, width=50,
+        self.template_name_txtbox = self.TextBox(
+            frame, width=46,
             name='main_template_name_textbox',
             textvariable=self.template_name_var
-        ).grid(row=0, column=1, sticky=tk.W)
+        )
+        self.template_name_txtbox.grid(row=0, column=1, sticky=tk.W)
+
+        self.lookup_btn = self.Button(
+            frame, text='Lookup',
+            name='main_lookup_btn',
+            command=callback_lookup_btn,
+            width=btn_width
+        )
+
+        self.close_lookup_btn = self.Button(
+            frame, text='Close',
+            name='main_close_lookup_btn',
+            command=self.search_chkbox.invoke,
+            width=btn_width
+        )
 
         # Robotframework button
         # rf_btn = self.Button(self.entry_frame, text='RF',
@@ -1732,11 +1894,12 @@ class Application:
             command=callback_app_backup_save_btn,
             width=btn_width
         ).pack(side=tk.LEFT)
-        self.Button(
+        self.close_backup_btn = self.Button(
             frame, text='Close',
             command=self.shift_to_main_app,
             width=btn_width
-        ).pack(side=tk.LEFT)
+        )
+        self.close_backup_btn.pack(side=tk.LEFT)
 
     def build_result(self):
         """Build result text"""
